==> flutter_mylibrary-0.6.34.tar.gz: lib/base/base_view_state.dart <==
import 'package:flutter/cupertino.dart';
import 'package:flutter_mylibrary/base/request_info.dart';
import 'package:flutter_mylibrary/config/const.dart';
import 'package:flutter_mylibrary/flutter_mylibrary.dart';
import 'package:flutter_mylibrary/http/easy_dio.dart';
import 'package:flutter_mylibrary/http/exception/http_error.dart';
import 'package:flutter_mylibrary/http/http_data.dart';
import 'package:flutter_mylibrary/utils/utils.dart';
import 'package:flutter_mylibrary/widget/widget_template.dart';

abstract class BaseViewState{

  /// 防止页面销毁后,异步任务才完成,导致报错
  bool disposed = false;

  /// 当前的页面状态,默认为busy,可在viewModel的构造方法中指定;
  ViewState viewState = ViewState.busy;

  EasyRefreshController _refreshController = EasyRefreshController();

  set refreshController(EasyRefreshController easyRefreshController){
    this._refreshController = easyRefreshController;
  }

  ///获取请求的数据，一般用于简单的页面请求
  RequestInfo getRequestInfo();

  void doOther(){}

  void refreshData(){}

  EasyRefreshController get refreshController => _refreshController;

  bool isInit = false;


  ///重新请求当报错时
  void requestWhenError(){
    setBusy();
    refreshData();
  }

  ///---------------------------------------------页面状态------------------------------------------------------

  HttpError httpError;

  bool get isBusy => viewState == ViewState.busy;

  bool get isIdle => viewState == ViewState.idle;

  bool get isError => viewState == ViewState.error;



  ///---------------------------------------------设置页面状态------------------------------------------------------


  void setIdle();

  void setBusy();

  void setError(HttpError httpError);

  void onError(HttpError httpError) {}


  ///---------------------------------------------通用请求方法------------------------------------------------------

  ///通常有下拉刷新请求，第一页之后不会再设置页面错误状态
  Future<HttpData<T>> request<T>(
      handleHttpData(HttpData data, RequestInfo requestInfo),
      {bool isSetError = true}
  ) async{

    doOther();
    RequestInfo requestInfo = getRequestInfo();
    HttpData<T> httpData = await doRequest<T>(requestInfo);

    _handleHttpData<T>(
        handleHttpData,
        httpData,
        requestInfo,
        isSetError
    );
    return httpData;
  }


  Future<HttpData<T>> requestList<T, E>(
      handleHttpData(HttpData data, RequestInfo requestInfo),
      {bool isSetError = true}
  ) async{

    doOther();
    RequestInfo requestInfo = getRequestInfo();
    HttpData<T> httpData = await doRequestList<T, E>(requestInfo);

    _handleHttpData<T>(
        handleHttpData,
        httpData,
        requestInfo,
        isSetError
    );
    return httpData;
  }





  ///处理返回的httpData
  void _handleHttpData<T>(
      handleHttpData(HttpData httpData, RequestInfo<T> requestInfo),
      HttpData httpData,
      RequestInfo requestInfo,
      bool isSetError
  ){
    if(isEmpty(httpData.httpError)){
      try{
        handleHttpData(httpData, requestInfo);
      }catch(e){
        setError(HttpError.parseDataError(e));
      }
    }else{
      if(isSetError){
        setError(httpData.httpError);
      }
    }
  }


  Future<HttpData<T>> doRequest<T>(RequestInfo<T> requestInfo) async{
    HttpData httpData = await EasyDio().request<T>(
      url: requestInfo?.url,
      method: requestInfo?.method,
      data: requestInfo?.data,
      options: requestInfo?.options,
      cacheKey: requestInfo?.cacheKey,
      isFormData: requestInfo?.isFormData,
      contentType: requestInfo?.contentType,
      publicMethod: requestInfo?.publicMethod,
      cancelToken: requestInfo?.cancelToken,
      onSuccess: requestInfo?.onSuccess,
      onStart: requestInfo?.onStart,
      onError: requestInfo?.onError,
      onFinish: requestInfo?.onFinish,
      validateResponse: requestInfo?.validateResponse,
      successCode: requestInfo?.successCode,
      codeKey: requestInfo?.codeKey,
      msgKey: requestInfo?.msgKey,
      onReceiveProgress: requestInfo?.onReceiveProgress,
      onSendProgress: requestInfo?.onSendProgress,
      handleRes: requestInfo?.handleRes,
      validateError: requestInfo?.validateError,
      checkDataEmpty: requestInfo?.checkDataEmpty
    );
    return httpData;
  }


  Future<HttpData<T>> doRequestList<T, E>(RequestInfo requestInfo) async{
    HttpData httpData = await EasyDio().requestList<T, E>(
      url: requestInfo?.url,
      method: requestInfo?.method,
      data: requestInfo?.data,
      options: requestInfo?.options,
      cacheKey: requestInfo?.cacheKey,
      isFormData: requestInfo?.isFormData,
      contentType: requestInfo?.contentType,
      publicMethod: requestInfo?.publicMethod,
      cancelToken: requestInfo?.cancelToken,
      onSuccess: requestInfo?.onSuccess,
      onStart: requestInfo?.onStart,
      onError: requestInfo?.onError,
      onFinish: requestInfo?.onFinish,
      validateResponse: requestInfo?.validateResponse,
      successCode: requestInfo?.successCode,
      codeKey: requestInfo?.codeKey,
      msgKey: requestInfo?.msgKey,
      onReceiveProgress: requestInfo?.onReceiveProgress,
      onSendProgress: requestInfo?.onSendProgress,
      handleRes: requestInfo?.handleRes,
      validateError: requestInfo?.validateError,
      checkDataEmpty: requestInfo?.checkDataEmpty
    );

    return httpData;
  }


///------------------------------------------------base下拉刷新控件----------------------------------------------------------

  Widget buildViewStateEasyRefresh<T>({
    T data,
    ScrollController scrollController,
    ViewStateModelWidgetListBuilder<T> widgetListBuilder,
    ViewStateModelWidgetBuilder<T> widgetBuilder,
    Widget busyWidget,
    Widget footWidget,
    ViewStateWidgetBuilder emptyWidgetBuilder,
    ViewStateWidgetBuilder errorWidgetBuilder,
    bool showErrorWidget = true,
    bool showEmptyWidget = true,
    bool showBusyWidget = true,
    Function onLoad,
    Function onRefresh,
  }){

    List<Widget> widgetList;
    Widget widget;


    if(isNotEmpty(widgetListBuilder)) {
      if(isBusy || isError){
        widgetList = [
          SliverToBoxAdapter(
            child: Container(),
          )
        ];
      }else{
        widgetList = widgetListBuilder(data);
      }
    }else if(isNotEmpty(widgetBuilder)){
      if(isBusy || isError){
        widget = Container();
      }else{
        widget = widgetBuilder(data);
      }
    }


    if(isEmpty(widgetList) && isEmpty(widget)){
      return Container();
    }


    if(isEmpty(footWidget)){
      return buildBaseViewStateEasyRefresh(
          scrollController: scrollController,
          onLoad: onLoad,
          emptyWidgetBuilder: emptyWidgetBuilder,
          errorWidgetBuilder: errorWidgetBuilder,
          onRefresh: onRefresh,
          widget: widget,
          widgetList: widgetList
      );
    }else{
      return Column(
        crossAxisAlignment: CrossAxisAlignment.start,
        children: [
          Expanded(
            child: buildBaseViewStateEasyRefresh(
                scrollController: scrollController,
                onLoad: onLoad,
                emptyWidgetBuilder: emptyWidgetBuilder,
                errorWidgetBuilder: errorWidgetBuilder,
                onRefresh: onRefresh,
                widget: widget,
                widgetList: widgetList
            ),
          ),
          (isNotEmpty(footWidget) && isIdle)? footWidget : Container()
        ],
      );
    }
  }

  ///底层建立下拉刷新
  Widget buildBaseViewStateEasyRefresh({
    ScrollController scrollController,
    Widget busyWidget,
    Widget footWidget,
    ViewStateWidgetBuilder emptyWidgetBuilder,
    ViewStateWidgetBuilder errorWidgetBuilder,
    bool showErrorWidget = true,
    bool showEmptyWidget = true,
    bool showBusyWidget = true,
    Function onLoad,
    Function onRefresh,
    List<Widget> widgetList,
    Widget widget,
  }){


    if(isNotEmpty(widgetList)) {
      if(isBusy || isError){
        widgetList = [
          SliverToBoxAdapter(
            child: Container(),
          )
        ];
      }
    }else if(isNotEmpty(widget)){
      if(isBusy || isError){
        widget = Container();
      }
    }


    Widget emptyErrorWidget = buildRefreshNoDataWidget(
        viewState: this,
        busyWidget: busyWidget,
        emptyWidgetBuilder: emptyWidgetBuilder,
        errorWidgetBuilder: errorWidgetBuilder,
        showErrorWidget: showErrorWidget,
        showEmptyWidget: showEmptyWidget,
        showBusyWidget: showBusyWidget
    );


    bool useSliverWidget;

    if(isNotEmpty(widgetList)){
      useSliverWidget = true;
    }else{
      useSliverWidget = false;
    }



    return useSliverWidget? EasyRefresh.custom(
      slivers: widgetList,
      scrollController: scrollController,
      enableControlFinishRefresh: true,
      enableControlFinishLoad: true,
      taskIndependence: true,
      controller: refreshController,
      emptyWidget: emptyErrorWidget,
      onLoad: isBusy? null : isEmpty(onLoad)? null :  () async{
        onLoad.call();
      },
      onRefresh: isBusy? null : () async{
        onRefresh.call();
      },
    ) : EasyRefresh(
      enableControlFinishLoad: true,
      enableControlFinishRefresh: true,
      scrollController: scrollController,
      emptyWidget: emptyErrorWidget,
      taskIndependence: true,
      child: widget,
      controller: refreshController,
      onRefresh: isBusy? null : () async{
        onRefresh.call();
      },
      onLoad: isBusy? null : isEmpty(onLoad)? null :  () async{
        onLoad.call();
      },
    );
  }
}



/// 页面状态类型
enum ViewState {
  idle,
  busy,
  error
}

==> flutter_mylibrary-0.6.46.tar.gz: lib/base/base_view_state.dart <==
import 'package:flutter/cupertino.dart';
import 'package:flutter_mylibrary/base/request_info.dart';
import 'package:flutter_mylibrary/config/const.dart';
import 'package:flutter_mylibrary/flutter_mylibrary.dart';
import 'package:flutter_mylibrary/http/easy_dio.dart';
import 'package:flutter_mylibrary/http/exception/http_error.dart';
import 'package:flutter_mylibrary/http/http_data.dart';
import 'package:flutter_mylibrary/utils/utils.dart';
import 'package:flutter_mylibrary/widget/widget_template.dart';

abstract class BaseViewState{

  /// 防止页面销毁后,异步任务才完成,导致报错
  bool disposed = false;

  /// 当前的页面状态,默认为busy,可在viewModel的构造方法中指定;
  ViewState viewState = ViewState.busy;

  EasyRefreshController _refreshController = EasyRefreshController();

  set refreshController(EasyRefreshController easyRefreshController){
    this._refreshController = easyRefreshController;
  }

  ///获取请求的数据，一般用于简单的页面请求
  RequestInfo getRequestInfo();

  void doOther(){}

  void refreshData(){}

  EasyRefreshController get refreshController => _refreshController;

  bool isInit = false;


  ///重新请求当报错时
  void requestWhenError(){
    setBusy();
    refreshData();
  }

  ///---------------------------------------------页面状态------------------------------------------------------

  HttpError httpError;

  bool get isBusy => viewState == ViewState.busy;

  bool get isIdle => viewState == ViewState.idle;

  bool get isError => viewState == ViewState.error;



  ///---------------------------------------------设置页面状态------------------------------------------------------


  void setIdle();

  void setBusy();

  void setError(HttpError httpError);

  void onError(HttpError httpError) {}


  ///---------------------------------------------通用请求方法------------------------------------------------------

  Future<HttpData<T>> request<T>(
      handleHttpData(HttpData data, RequestInfo requestInfo),
      {bool isSetError = true}
  ) async{

    doOther();
    RequestInfo requestInfo = getRequestInfo();
    HttpData<T> httpData = await doRequest<T>(requestInfo);

    _handleHttpData<T>(
        handleHttpData,
        httpData,
        requestInfo,
        isSetError
    );
    return httpData;
  }


  ///通常有下拉刷新请求，第一页之后不会再设置页面错误状态
  Future<HttpData<T>> requestList<T, E>(
      handleHttpData(HttpData data, RequestInfo requestInfo),
      {bool isSetError = true}
  ) async{

    doOther();
    RequestInfo requestInfo = getRequestInfo();
    HttpData<T> httpData = await doRequestList<T, E>(requestInfo);

    _handleHttpData<T>(
        handleHttpData,
        httpData,
        requestInfo,
        isSetError
    );
    return httpData;
  }





  ///处理返回的httpData
  void _handleHttpData<T>(
      handleHttpData(HttpData httpData, RequestInfo requestInfo),
      HttpData httpData,
      RequestInfo requestInfo,
      bool isSetError
  ){
    if(isEmpty(httpData.httpError)){
      try{
        handleHttpData(httpData, requestInfo);
      }catch(e){
        setError(HttpError.parseDataError(e));
      }
    }else{
      if(isSetError){
        setError(httpData.httpError);
      }
    }
  }


  Future<HttpData<T>> doRequest<T>(RequestInfo requestInfo) async{
    HttpData httpData = await EasyDio().request<T>(
      url: requestInfo?.url,
      method: requestInfo?.method,
      data: requestInfo?.data,
      options: requestInfo?.options,
      cacheKey: requestInfo?.cacheKey,
      isFormData: requestInfo?.isFormData,
      contentType: requestInfo?.contentType,
      publicMethod: requestInfo?.publicMethod,
      cancelToken: requestInfo?.cancelToken,
      onSuccess: requestInfo?.onSuccess,
      onStart: requestInfo?.onStart,
      onError: requestInfo?.onError,
      onFinish: requestInfo?.onFinish,
      validateResponse: requestInfo?.validateResponse,
      successCode: requestInfo?.successCode,
      codeKey: requestInfo?.codeKey,
      msgKey: requestInfo?.msgKey,
      onReceiveProgress: requestInfo?.onReceiveProgress,
      onSendProgress: requestInfo?.onSendProgress,
      handleRes: requestInfo?.handleRes,
      validateError: requestInfo?.validateError,
      checkDataEmpty: requestInfo?.checkDataEmpty
    );
    return httpData;
  }


  Future<HttpData<T>> doRequestList<T, E>(RequestInfo requestInfo) async{
    HttpData httpData = await EasyDio().requestList<T, E>(
      url: requestInfo?.url,
      method: requestInfo?.method,
      data: requestInfo?.data,
      options: requestInfo?.options,
      cacheKey: requestInfo?.cacheKey,
      isFormData: requestInfo?.isFormData,
      contentType: requestInfo?.contentType,
      publicMethod: requestInfo?.publicMethod,
      cancelToken: requestInfo?.cancelToken,
      onSuccess: requestInfo?.onSuccess,
      onStart: requestInfo?.onStart,
      onError: requestInfo?.onError,
      onFinish: requestInfo?.onFinish,
      validateResponse: requestInfo?.validateResponse,
      successCode: requestInfo?.successCode,
      codeKey: requestInfo?.codeKey,
      msgKey: requestInfo?.msgKey,
      onReceiveProgress: requestInfo?.onReceiveProgress,
      onSendProgress: requestInfo?.onSendProgress,
      handleRes: requestInfo?.handleRes,
      validateError: requestInfo?.validateError,
      checkDataEmpty: requestInfo?.checkDataEmpty
    );

    return httpData;
  }


///------------------------------------------------base下拉刷新控件----------------------------------------------------------

  Widget buildViewStateEasyRefresh<T>({
    T data,
    ScrollController scrollController,
    ViewStateModelWidgetListBuilder<T> widgetListBuilder,
    ViewStateModelWidgetBuilder<T> widgetBuilder,
    Widget busyWidget,
    Widget footWidget,
    ViewStateWidgetBuilder emptyWidgetBuilder,
    ViewStateWidgetBuilder errorWidgetBuilder,
    bool showErrorWidget = true,
    bool showEmptyWidget = true,
    bool showBusyWidget = true,
    Function onLoad,
    Function onRefresh,
  }){

    List<Widget> widgetList;
    Widget widget;


    if(isNotEmpty(widgetListBuilder)) {
      if(isBusy || isError){
        widgetList = [
          SliverToBoxAdapter(
            child: Container(),
          )
        ];
      }else{
        widgetList = widgetListBuilder(data);
      }
    }else if(isNotEmpty(widgetBuilder)){
      if(isBusy || isError){
        widget = Container();
      }else{
        widget = widgetBuilder(data);
      }
    }


    if(isEmpty(widgetList) && isEmpty(widget)){
      return Container();
    }


    if(isEmpty(footWidget)){
      return buildBaseViewStateEasyRefresh(
          scrollController: scrollController,
          onLoad: onLoad,
          emptyWidgetBuilder: emptyWidgetBuilder,
          errorWidgetBuilder: errorWidgetBuilder,
          onRefresh: onRefresh,
          widget: widget,
          widgetList: widgetList
      );
    }else{
      return Column(
        crossAxisAlignment: CrossAxisAlignment.start,
        children: [
          Expanded(
            child: buildBaseViewStateEasyRefresh(
                scrollController: scrollController,
                onLoad: onLoad,
                emptyWidgetBuilder: emptyWidgetBuilder,
                errorWidgetBuilder: errorWidgetBuilder,
                onRefresh: onRefresh,
                widget: widget,
                widgetList: widgetList
            ),
          ),
          (isNotEmpty(footWidget) && isIdle)? footWidget : Container()
        ],
      );
    }
  }

  ///底层建立下拉刷新
  Widget buildBaseViewStateEasyRefresh({
    ScrollController scrollController,
    Widget busyWidget,
    Widget footWidget,
    ViewStateWidgetBuilder emptyWidgetBuilder,
    ViewStateWidgetBuilder errorWidgetBuilder,
    bool showErrorWidget = true,
    bool showEmptyWidget = true,
    bool showBusyWidget = true,
    Function onLoad,
    Function onRefresh,
    List<Widget> widgetList,
    Widget widget,
  }){


    if(isNotEmpty(widgetList)) {
      if(isBusy || isError){
        widgetList = [
          SliverToBoxAdapter(
            child: Container(),
          )
        ];
      }
    }else if(isNotEmpty(widget)){
      if(isBusy || isError){
        widget = Container();
      }
    }


    Widget emptyErrorWidget = buildRefreshNoDataWidget(
        viewState: this,
        busyWidget: busyWidget,
        emptyWidgetBuilder: emptyWidgetBuilder,
        errorWidgetBuilder: errorWidgetBuilder,
        showErrorWidget: showErrorWidget,
        showEmptyWidget: showEmptyWidget,
        showBusyWidget: showBusyWidget
    );


    bool useSliverWidget;

    if(isNotEmpty(widgetList)){
      useSliverWidget = true;
    }else{
      useSliverWidget = false;
    }



    return useSliverWidget? EasyRefresh.custom(
      slivers: widgetList,
      scrollController: scrollController,
      enableControlFinishRefresh: true,
      enableControlFinishLoad: true,
      taskIndependence: true,
      controller: refreshController,
      emptyWidget: emptyErrorWidget,
      onLoad: isBusy? null : isEmpty(onLoad)? null :  () async{
        onLoad.call();
      },
      onRefresh: isBusy? null : () async{
        onRefresh.call();
      },
    ) : EasyRefresh(
      enableControlFinishLoad: true,
      enableControlFinishRefresh: true,
      scrollController: scrollController,
      emptyWidget: emptyErrorWidget,
      taskIndependence: true,
      child: widget,
      controller: refreshController,
      onRefresh: isBusy? null : () async{
        onRefresh.call();
      },
      onLoad: isBusy? null : isEmpty(onLoad)? null :  () async{
        onLoad.call();
      },
    );
  }
}



/// 页面状态类型
enum ViewState {
  idle,
  busy,
  error
}
